searchData={"items":[{"type":"module","title":"Hush","doc":"Extensible runtime configuration loader with pluggable providers","ref":"Hush.html"},{"type":"function","title":"Hush.release_mode?/0","doc":"Is the app running in release mode?","ref":"Hush.html#release_mode?/0"},{"type":"function","title":"Hush.resolve!/0","doc":"Resolve configuration\n\nWhen called with resolve!/0 it will default to all loaded applications' configuration.\nWhen called with resolve!/1 with the configuration as an argument it will process that.","ref":"Hush.html#resolve!/0"},{"type":"function","title":"Hush.resolve!/2","doc":"","ref":"Hush.html#resolve!/2"},{"type":"module","title":"Hush.ConfigProvider","doc":"This config provider can be attached to your release to run automatically on boot.","ref":"Hush.ConfigProvider.html"},{"type":"behaviour","title":"Hush.Provider","doc":"Hush relies on providers to resolve values that the consumer requests.\n\nYou can [read here](https://hexdocs.pm/hush/readme.html#writing-your-own-provider) on how to write your own provider.","ref":"Hush.Provider.html"},{"type":"callback","title":"Hush.Provider.fetch/1","doc":"","ref":"Hush.Provider.html#c:fetch/1"},{"type":"function","title":"Hush.Provider.fetch/2","doc":"Fetch a value from a provider","ref":"Hush.Provider.html#fetch/2"},{"type":"callback","title":"Hush.Provider.load/1","doc":"","ref":"Hush.Provider.html#c:load/1"},{"type":"function","title":"Hush.Provider.valid?/1","doc":"Is the provider a valid one?","ref":"Hush.Provider.html#valid?/1"},{"type":"type","title":"Hush.Provider.child_spec/0","doc":"","ref":"Hush.Provider.html#t:child_spec/0"},{"type":"module","title":"Hush.Provider.SystemEnvironment","doc":"Provider to resolve runtime environmental variables","ref":"Hush.Provider.SystemEnvironment.html"},{"type":"module","title":"Hush.Resolver","doc":"Replace configuration with provider-aware resolvers.","ref":"Hush.Resolver.html"},{"type":"function","title":"Hush.Resolver.resolve/2","doc":"Substitute {:hush, Hush.Provider, \"key\", [options]} present in the the config argument.","ref":"Hush.Resolver.html#resolve/2"},{"type":"function","title":"Hush.Resolver.resolve!/2","doc":"Substitute {:hush, Hush.Provider, \"key\", [options]} present in the the config argument","ref":"Hush.Resolver.html#resolve!/2"},{"type":"behaviour","title":"Hush.Transformer","doc":"Hush relies on transformers to post process values after resolving them.\n\nYou can [read here](https://hexdocs.pm/hush/readme.html#writing-your-own-transformer) on how to write your own transformer.","ref":"Hush.Transformer.html"},{"type":"function","title":"Hush.Transformer.apply/2","doc":"","ref":"Hush.Transformer.html#apply/2"},{"type":"callback","title":"Hush.Transformer.key/0","doc":"","ref":"Hush.Transformer.html#c:key/0"},{"type":"callback","title":"Hush.Transformer.transform/2","doc":"","ref":"Hush.Transformer.html#c:transform/2"},{"type":"module","title":"Hush.Transformer.Apply","doc":"Apply function transformer","ref":"Hush.Transformer.Apply.html"},{"type":"module","title":"Hush.Transformer.Cast","doc":"Utility functions to cast strings to Elixir native types","ref":"Hush.Transformer.Cast.html"},{"type":"function","title":"Hush.Transformer.Cast.to!/2","doc":"Cast a string to any type.\nAn error is raised on failure.","ref":"Hush.Transformer.Cast.html#to!/2"},{"type":"module","title":"Hush.Transformer.ToFile","doc":"File writer transformer","ref":"Hush.Transformer.ToFile.html"},{"type":"extras","title":"Hush","doc":"# Hush\n\n[![Build Status](https://img.shields.io/github/actions/workflow/status/gordalina/hush/ci.yml?branch=main&style=flat-square)](https://github.com/gordalina/hush/actions/workflows/ci.yml)\n[![Coverage Status](https://img.shields.io/codecov/c/github/gordalina/hush?style=flat-square)](https://app.codecov.io/gh/gordalina/hush)\n[![hex.pm version](https://img.shields.io/hexpm/v/hush?style=flat-square)](https://hex.pm/packages/hush)\n[![hex.pm downloads](https://img.shields.io/hexpm/dt/hush?style=flat-square)]([LICENSE](https://hex.pm/packages/hush))\n\nHush is designed to help developers configure their applications at runtime and in release mode, retrieving configuration from multiple providers, without having to depend on secret files or hardcoded configuration.\n\nDocumentation can be found at [https://hexdocs.pm/hush](https://hexdocs.pm/hush).","ref":"readme.html"},{"type":"extras","title":"Overview - Hush","doc":"Hush can be used to inject configuration that is not known at compile time, such as environmental variables (e.g.: Heroku's PORT env var), sensitive credentials such as your database password, or any other information you need.\n\n```elixir\n# config/prod.exs\nalias Hush.Provider.{AwsSecretsManager, GcpSecretManager, SystemEnvironment}\n\nconfig :app, Web.Endpoint,\n  http: [port: {:hush, SystemEnvironment, \"PORT\", [cast: :integer]}]\n\nconfig :app, App,\n  cdn_url: {:hush, GcpSecretManager, \"CDN_DOMAIN\", [apply: &{:ok, \"https://\" <> &1}]}\n\nconfig :app, App.RedshiftRepo,\n  password: {:hush, AwsSecretsManager, \"REDSHIFT_PASSWORD\"}\n```\n\nHush resolves configuration from using providers, it ships with a `SystemEnvironment` provider which reads environmental variables, but multiple providers exist. You can also [write your own easily](#writing-your-own-provider).\n\n| Provider            | Description                                                                               | Link                                                            |\n| ------------------- | ----------------------------------------------------------------------------------------- | --------------------------------------------------------------- |\n| `SystemEnvironment` | Reads environmental variables.                                                            |                                                                 |\n| `AwsSecretsManager` | Load secrets from [AWS Secrets Manager](https://aws.amazon.com/secrets-manager/).         | [GitHub](https://github.com/gordalina/hush_aws_secrets_manager) |\n| `GcpSecretManager`  | Load secrets from [Google Cloud Secret Manager](https://cloud.google.com/secret-manager). | [GitHub](https://github.com/gordalina/hush_gcp_secret_manager)  |","ref":"readme.html#overview"},{"type":"extras","title":"Installation - Hush","doc":"Add `hush` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:hush, \"~> 1.1\"}\n  ]\nend\n```\n\nRun `mix deps.get` to install it.\n\nSome providers may need to initialize applications or even start processes to function correctly. The providers will be explicit about whether they need to be loaded at startup or not. `GcpSecretsManager` unlike `SystemEnvironment` is one such example. To load the provider you need to configure it like so. **Note:** `SystemEnvironment` does not need to be loaded at startup.\n\n```elixir\n# config/config.exs\n\nalias Hush.Providers.GcpSecretManager\n\nconfig :hush,\n  providers: [\n    GcpSecretManager\n  ]\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Hush","doc":"Hush can be loaded in two ways, at runtime in your application, or as a [Config.Provider](https://hexdocs.pm/elixir/Config.Provider.html) in release mode. A [sample app](https://github.com/gordalina/hush_sample_app) has been written so you can see how it's configured.","ref":"readme.html#usage"},{"type":"extras","title":"Loading at runtime - Hush","doc":"```elixir\n# application.ex\n\ndef start(_type, _args) do\n  Hush.resolve!()\nend\n```","ref":"readme.html#loading-at-runtime"},{"type":"extras","title":"Loading via in release mode - Hush","doc":"To load hush as a config provider, you need to define in your `releases` in `mix.exs`.\n\n```elixir\ndef project do\n  [\n    # ...\n    releases: [\n      app: [\n        config_providers: [{Hush.ConfigProvider, nil}]\n      ]\n    ]\n  ]\n  end\n```\n\nIf you are using Hush in runtime and release mode, make sure to only resolve configuration in non release mode:\n\n```elixir\n# application.ex\n\ndef start(_, _) do\n  unless Hush.release_mode?(), do: Hush.resolve!()\nend\n```","ref":"readme.html#loading-via-in-release-mode"},{"type":"extras","title":"Configuration format - Hush","doc":"Hush will resolve any tuple in the following format into a value.\n\n```elixir\n{:hush, Hush.Provider, \"key\", options \\\\ []}\n```\n\n- `Hush.Provider` can be any module that implements its behaviour.\n- `\"key\"` is passed to the provider to retrieve the data.\n- `options` is a a Keyword list with the following properties:\n  - `default: any()` - If the provider can't find the value, hush will return this value\n  - `optional: boolean()` - By default, Hush will raise an error if it cannot find a value and there's no default, unless you mark it as `optional`.\n  - `apply: fun(any()) :: {:ok, any()} | {:error, String.t()}` - Apply a function to the value resolved by Hush.\n  - `cast: :string | :atom | :charlist | :float | :integer | :boolean | :module` - You can ask Hush to cast the value to a Elixir native type.\n  - `to_file: string()` - Write the data to the path give in `to_file()` and return the path.\n\nAfter Hush resolves a value it runs them through Transfomers.","ref":"readme.html#configuration-format"},{"type":"extras","title":"Examples - Hush","doc":"By default if a given `key` is not found by the provider, Hush will raise an error. To prevent this, provide a `default` or `optional: true` in the `options` component of the tuple.\n\n#### Default\n\n```elixir\n# config/config.exs\nalias Hush.Provider.SystemEnvironment\n\nconfig :app,\n  url: {:hush, SystemEnvironment, \"HOST\", default: \"example.domain\"}\n\n# result without environmental variable\nassert \"example.domain\" == Application.get_env(:app, :url)\n\n# result with env HOST=production.domain\nassert \"production.domain\" == Application.get_env(:app, :url)\n```\n\n#### Casting\n\nHere we are reading the `PORT` environmental variable, casting it to an integer and returning it\n\n```elixir\n# config/config.exs\nalias Hush.Provider.SystemEnvironment\n\nconfig :app,\n  port: {:hush, SystemEnvironment, \"PORT\", cast: :integer, default: 4000}\n\n# result without environmental variable\nassert 4000 == Application.get_env(:app, :url)\n\n# result with env PORT=443\nassert 443 == Application.get_env(:app, :url)\n```\n\n#### Optional\n\n```elixir\n# config/dev.exs\nalias Hush.Provider.SystemEnvironment\n\nconfig :app,\n  can_be_nil: {:hush, SystemEnvironment, \"KEY\", optional: true}\n\n# result without environmental variable\nassert nil == Application.get_env(:app, :can_be_nil)\n\n# result with env KEY=\"is not nil\"\nassert \"is not nil\" == Application.get_env(:app, :can_be_nil)\n```","ref":"readme.html#examples"},{"type":"extras","title":"Concurrency & Cache - Hush","doc":"By default Hush will fetch secrets from providers concurrently and will save them to a short-lived cache to prevent fetching the same secret multiple times.\n\nThe defaults for these values are `System.schedulers_online/0` for concurrency and `5000ms` for concurrency timeouts.\n\nThese values can be modified with the following configuration:\n\n```elixir\n# config/config.exs\nconfig :hush,\n  max_concurrency: 10,\n  timeout: 5000, # milliseconds\n```","ref":"readme.html#concurrency-cache"},{"type":"extras","title":"Transfomers - Hush","doc":"By default Hush ships with the following transformers:\n\n- **Hush.Transfomer.Cast**: Takes an argument `cast` and converts a value into a specific type.\n- **Hush.Transfomer.ToFile**: Takes an arugment `to_file` and outputs the value into the path provided.\n\nIt is possible to add more transformers by the following configuration:\n\n```elixir\n# config/prod.exs\n\nalias Hush.Provider.SystemEnvironment\n\nconfig :hush,\n  transfomers: [\n    App.Hush.JsonToMapTransfomer\n  ]\n\nconfig :app,\n  allowed_urls: {:hush, SystemEnvironment, \"alloweds_urls\", [json: true]}\n```\n\nIt is also possible to override the transforms Hush will process, and the order they will execute in. See [below](#overriding-transformers) for more information.","ref":"readme.html#transfomers"},{"type":"extras","title":"Writing your own transfomer - Hush","doc":"The currently [shipped](https://github.com/gordalina/hush/blob/master/lib/transformer/cast.ex) [transfomers](https://github.com/gordalina/hush/blob/master/lib/transformer/to_file.ex) are good examples on how to implement transformers.\n\nTransformers are executed in order they are defined, first is `Cast`, next is `ToFile` and then the ones configured by you, e.g.:\n\n```elixir\n# config/prod.exs\n\nconfig :hush,\n  transformers: [\n    App.Hush.JsonTransformer\n  ]\n```\n\nLets dissect a transformer as an example. A transformer has to implement the `Hush.Transformer` behaviour, and as such it has to implement the `key/0` and `transform/2` functions.\n\nA transformer is going to be executed if a configuration tuple requests it by passing the value of `key/0` into its options. An example is seeing the `json` parameter being passed into the `value` configuration. Hush will process any transformers in which their `key/0` function returns `:json`.\n\nOnce a configuration tuple requests a transfomer, a function `transform/2` is called, where the first argument is what is passed as a value of the `key/0` (in the example below it would be `:abort_on_failure`), and the second argument would be the current value returned by the provider transformed by any previous transformers.\n\n```elixir\n# config/prod.exs\n\nconfig :app,\n  value: {:hush, SystemEnvironment, \"key\", [json: :abort_on_failure]}\n```\n\n```elixir\n# lib/app/hush/JsonTransformer.ex\n\ndefmodule App.Hush.JsonTransformer do\n  @behaviour Hush.Transformer\n\n  @impl true\n  @spec key() :: :json\n  def key(), do :json\n\n  @impl true\n  @spec transform(config :: any(), value :: any()) :: {:ok, any()} | {:error, String.t()}\n  def transform(config, value) do\n    try do\n      Jason.decode!(value)\n    rescue\n      error ->\n        case config do\n          :abort_on_failure ->\n            {:error, \"Couldn't convert #{value} to json: #{error.message}\"}\n          _ ->\n            {:ok, nil}\n        end\n    end\n  end\nend\n```","ref":"readme.html#writing-your-own-transfomer"},{"type":"extras","title":"Overriding Transformers - Hush","doc":"The following example woud take a value passed as an environment variable `ALLOWED_URLS='[\"http://example.com\"]'` into a file named `/tmp/urls.json` with the contents `[\"https://example.com\"]`, all due to the order in which the transformers are executed and the fact that `override_transformers` is `true`.\n\n```elixir\n# config/prod.exs\n\nconfig :hush,\n  override_transformers: true,\n  transformers: [\n    Hush.Transformer.Cast,\n    App.Hush.HttpToHttpsTransformer,\n    App.Hush.JsonTransformer,\n    Hush.Transformer.ToFile,\n  ]\n\nconfig :app,\n  value: {:hush, SystemEnvironment, \"ALLOWED_URLS\", [http_to_https: true, json: true, to_file: \"/tmp/urls.json\" ]}\n```\n\n```elixir\n# lib/app/hush/HttpToHttpsTransfomer.ex\n\ndefmodule App.HttpToHttpsTransfomer do\n  @behaviour Hush.Transformer\n\n  @impl true\n  @spec key() :: :http_to_https\n  def key(), do :http_to_https\n\n  @impl true\n  @spec transform(config :: any(), value :: any()) :: {:ok, any()} | {:error, String.t()}\n  def transfomer(_config, value) do\n    {:ok, Enum.map(value, &http_to_https(&2))}\n  end\n\n  def http_to_https(value) do\n    Regex.replace(~r/^http:/, value, \"https:\")\n  end\nend\n```","ref":"readme.html#overriding-transformers"},{"type":"extras","title":"Providers - Hush","doc":"","ref":"readme.html#providers"},{"type":"extras","title":"Writing your own provider - Hush","doc":"An example provider is `Hush.Provider.SystemEnvironment`, which reads\nenvironmental variables at runtime. Here's an example of how that provider\nwould look in a app configuration.\n\n```elixir\n  alias Hush.Provider.SystemEnvironment\n\n  config :app, Web.Endpoint,\n    http: [port: {:hush, SystemEnvironment, \"PORT\", [cast: :integer, default: 4000]}]\n```\n\nThis behaviour expects two functions:\n\n- ```elixir\n  load(config :: Keyword.t()) :: :ok | {:ok, [child_spec()]} | {:error, any()}\n  ```\n\n  This function is called at startup time, here you can perform any initialization you need, such as loading applications that you depend on. If you need to startup any processes, you can return a list of `child_spec()` which will be brought up by Hush's supervisor and brought down after hush runs.\n\n- ```elixir\n  fetch(key :: String.t()) :: {:ok, String.t()} | {:error, :not_found} | {:error, any()}\n  ```\n\n  This function is called when hush is resolving a key with you provider.\n  Ensure that you implement a `{:error, :not_found}` if the value can't be found as hush will replace with it a default one if the user providede one.\n\n  Note: All values are required by default, so if the user did not supply a default or made it optional, hush will trigger the error, you don't need to handle that use-case.\n\nTo implement that provider we can use the following code.\n\n```elixir\n  defmodule Hush.Provider.SystemEnvironment do\n  @moduledoc \"\"\"\n  Provider to resolve runtime environmental variables\n  \"\"\"\n\n  @behaviour Hush.Provider\n\n  @impl Hush.Provider\n  @spec load(config :: Keyword.t()) :: :ok | {:ok, [child_spec()]} | {:error, any()}\n  def load(_config), do: :ok\n\n  @impl Hush.Provider\n  @spec fetch(key :: String.t()) :: {:ok, String.t()} | {:error, :not_found}\n  def fetch(key) do\n    case System.get_env(key) do\n      nil -> {:error, :not_found}\n      value -> {:ok, value}\n    end\n  end\nend\n```","ref":"readme.html#writing-your-own-provider"},{"type":"extras","title":"Compatibility - Hush","doc":"| Hush | Erlang/OTP | Elixir |\n| - | - | - |\n| `>= 1.0.0` | `>= 21.0.0` | `>= 1.10.0` |\n| `<= 0.5.0` | `>= 20.0.0` | `>= 1.9.0` |","ref":"readme.html#compatibility"},{"type":"extras","title":"License - Hush","doc":"Hush is released under the Apache License 2.0 - see the [LICENSE](LICENSE) file.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v1.1.0 - Changelog","doc":"- Add support for Elixir 1.17 and OTP 27\n- Drop support for Elixir 1.10","ref":"changelog.html#v1-1-0"},{"type":"extras","title":"v1.0.2 - Changelog","doc":"- [#9](https://github.com/gordalina/hush/pull/9) Use Exception.message to retrieve error message. [@martosaur](https://github.com/martosaur)\n- [#10](https://github.com/gordalina/hush/pull/10) Do not buffer cache results in Task.async_stream. [@jramos](https://github.com/jramos)","ref":"changelog.html#v1-0-2"},{"type":"extras","title":"v1.0.1 - Changelog","doc":"- Display a warning when provider fetching times out","ref":"changelog.html#v1-0-1"},{"type":"extras","title":"v1.0.0 - Changelog","doc":"- Add support to load processes from providers.\n- Add asynchronous secret loading.\n- Prevent secrets to be fetched more than once.\n- Fixed [#7](https://github.com/gordalina/hush/issues/7) where a provider error would raise an error rather than returning a good error message.","ref":"changelog.html#v1-0-0"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"- [#2](https://github.com/gordalina/hush/pull/2) Add Apply Transformer\n\n  ```ex\n    config :app, Web.Endpoint,\n      cdn_url: {:hush, GcpSecretManager, \"CDN_DOMAIN\", [apply: &{:ok, \"https://\" <> &1}]}\n  ```\n- Add Elixir 1.12 and OTP/24 compatibility\n- Minor CI changes & documentation updates","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"- Add hush_aws_secrets_manager\n- Add Elixir 1.11 compatibility\n- Documentation updates","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"- Add Transformers to mutate data in runtime.","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.2 - Changelog","doc":"- Add `to_file: String.t()` as an option to pipe secret data into a file.","ref":"changelog.html#v0-3-2"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"- Bug: Traversing non-iterable structs trips the resolver when running as Config.Provider","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"- Add configuration resolving to lists, maps & tuples\n- Better error messages on resolve failure.","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"- Maps can be resolved","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"- Announced public release","ref":"changelog.html#v0-1-0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}